/*
   AS3935 Lightning Detector
  Written by Glen Popiel - KW5GP

  Based on the Sparkfun AS3935 library sketches by Elias Santistevan.

  Sparkfun sketches by: Elias Santistevan
  SparkFun Electronics
  Date: April, 2019
  License: This code is public domain but you buy me a beer if you use this and we meet someday (Beerware license).

*/

//#define debug // Startup, calibration and basic debug
//#define debug1  // Primarily timekeeping and display debug
//#define debug2  // Lightning detection and interrupt processing debug

bool fail_test = true;  // variable use to trigger a failure to test error handling code

#include <SPI.h>
#include <SparkFun_AS3935.h>
#include <Adafruit_GFX.h> // The Adafruit Graphics Library
#include <Adafruit_ILI9341.h> // The ILI9341 TFT Display Library

#define baud_rate 115200
#define tft_cyan ILI9341_CYAN
#define tft_black ILI9341_BLACK
#define tft_red ILI9341_RED
#define tft_green ILI9341_GREEN
#define tft_yellow ILI9341_YELLOW

#define INDOOR 0x12
#define OUTDOOR 0xE
#define LIGHTNING_INT 0x08
#define DISTURBER_INT 0x04
#define NOISE_INT 0x01
#define ANTFREQ 3
#define LD_Tune_jumper 4
#define LD_relay 3
#define IRQ_pin 2
#define TFT_CS 9
#define TFT_DC 6
#define LD_bypass 5
#define tft_delay 100
#define settle 100  // Time to allow the AS3935 IRQ line to settle after a read
#define clear_alert_time 3600 // # of ticks before clearing an alert
#define start_delay 10000 // milliseconds to delay start to avoid false triggering while initial statistics are gathered

long hold_down; // Timer used to prevent false triggers after startup and bypass switch changes
int hold_down_time = 1000; // Timer to ignore interrupts in milliseconds

long last_event = 0;  // How long ago the last event occurred
long last_scan = 0;
bool active_alert = false;
bool new_event = false;
bool disconnected = false;
bool LD_bypassed = false; // The status of the bypass/reset switch
bool bypass_active = false;
String event_time = "";
int tft_event_background = tft_black;
int text_spacing; // Use to space the timestamp line
byte distance;  // Lightning Distance
long lightEnergy; // Lightning Energy
bool auto_clear = true; // Automatically clear alert after clear_alert_time

// The lightning detector defaults to an indoor setting (higher
// gain/sensitivity)

bool LD_mode = INDOOR;

byte Interrupt_type = 99; // Since 0 is a valid interrupt type, we use 99 to tell us there are no interrupts

byte intVal = 0; // Value of the Interrupt Register
String int_text = "";

// "Disturbers" are events that are false lightning events. If you find
// yourself seeing a lot of disturbers you can have the chip not report those
// events on the interrupt lines.

bool LD_Disturber = true; // False = Enabled, True = Disabled (Masked)

// Values for modifying the Lightning Detector's settings.
// All of these values are set to default values.

// Noise floor setting from 1-7, one being the lowest.
// Default setting is two.

byte noiseFloor = 3;

// Watchdog threshold setting can be from 1-10, one being the lowest.
// Default setting is two.

byte watchDogVal = 2;

// Spike Rejection setting from 1-11, one being the lowest. Default setting is
// two. The shape of the spike is analyzed during the chip's
// validation routine. You can round this spike at the cost of sensitivity to
// distant events.

byte spike = 2;

// This setting will change when the lightning detector issues an interrupt.
// For example you will only get an interrupt after five lightning strikes
// instead of one. Default is one, and it takes settings of 1, 5, 9 and 16.

byte lightningThresh = 1;

#define font_size 4  // The CW text display font size
#define status_font_size 2  // The font size for the status line

// Here you give the value of the capacitor you want turned on. It accepts up
// to 120pF in steps of 8pF: 8, 16, 24, 32 etc.The change in frequency is
// somewhat modest. At the maximum value you can lower the frequency up to 22kHz.

// As a starting point, the Sparkfun products designed in house ship around 496kHz
// (though of course every board is different) putting you within one percent
// of a perfect resonance; the datasheet specifies being within 3.5 percent as
// optimal. You can leave it at zero if using auto-tune jumper and let the sketch determine the
// proper capacitor value.

int tuning_cap = 16; // Set the value to the correct capacitance in pF if not using auto-tune jumper - must be a valid capacitance

long freq = 0;
long end_time = 0;
long tick = 0;

bool osc_fail;

// Chip select for Lightning Detector SPI on pin ten.
int spiCS = 10;
int tuneVal;
int best_cap = 16;
float best_diff = 0;
float diff;
long last_sim = 0; // timer for simulated interrupts
long test_tick = 0; // 10 second tick for simulated interrupts
int trigger_time = 60; // Set the sim trigger time to 60 seconds
bool first_pass = true;

// SPI
SparkFun_AS3935 lightning;

// Use hardware SPI (on Uno/Nano, #13, #12, #11) and the above for CS/DC
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);  // Create the TFT instance

void setup()
{

#if defined (debug) || defined(debug1) || defined (debug2)  // Only start the Serial port if we're debugging
  Serial.begin(baud_rate);
  Serial.println("Starting Serial Port for debugging");
#endif

  start_tft();

  pinMode(IRQ_pin, INPUT); // The Detector IRQ input pin
  pinMode(LD_bypass, INPUT_PULLUP); // The disconnect relay bypass/alert reset switch
  pinMode(LD_relay, OUTPUT); // The Relay drive output pin
  pinMode(LD_Tune_jumper, INPUT_PULLUP);  // The Tuning_jumper input pin

  SPI.begin(); // Start the SPI bus

  check_LD_module();  // Check to see if the lightning detector module is basically functional

  check_calibration();  // Run/Set the LD module calibration

  clear_display();

  update_status();

  // Read the Lightning detector to clear any interrupts before starting
  intVal = lightning.readInterruptReg();
  intVal = 99;

#ifdef debug2
  Serial.println("Reading Detector IRQ to reset");
#endif

#if defined(debug) || defined(debug2)
  Serial.println("**** Detector Ready ****");
#endif

  tft.setTextSize(2);

  bypass_active = true; // Used to force bypass check and LD bypass reset process on first pass
  active_alert = false;
  hold_down = millis() + hold_down_time + start_delay;
}

void loop()
{

#ifdef debug2
  //  simulate_interrupt(); // Debug code to semi-randomly simulate an AS3935 interrupt
#endif

  poll_interrupt(); // Check the interrupt pin

  // read the bypass switch
  LD_bypassed = !(digitalRead(LD_bypass));  // Read the bypass/reset switch

  if (LD_bypassed)
  {
    // Bypass switch is enabled
    tft_event_background = tft_yellow;
    if (!bypass_active || first_pass)
    {
      hold_down = millis() + hold_down_time;
      if (disconnected)
      {
        // Reconnect the Antenna
        digitalWrite(LD_relay, LOW);
        disconnected = false;
      }
      // Update the screen
      tft_event_background = tft_yellow;
      clear_text_area(tft_event_background);
      tft.setTextColor(tft_black, tft_event_background);
      tft.setCursor(35, 70);

      tft.print("No Lightning Detected");
      tft.setCursor(80, 120);

      tft.print("Bypass Enabled");
      bypass_active = true;
      active_alert = false;
      first_pass = false;

#ifdef debug2
      Serial.println("first pass off");
#endif
    }
  } else
  {
    if (!bypass_active)
    {

    } else
    {
      // The bypass switch has been turned off, reset alerts, etc
#ifdef debug2
      Serial.println("Bypass turned off");
#endif
      hold_down = millis() + hold_down_time;
      bypass_active = false;

      tft_event_background = tft_green;
      clear_text_area(tft_event_background);
      tft.setTextColor(tft_black, tft_event_background);
      tft.setCursor(35, 70);

      tft.print("No Lightning Detected");
      tft.setCursor(60, 140);

      tft.print("Antenna Connected");
      active_alert = false;

#ifdef debug2
      Serial.println("first pass off");
#endif
    }
    first_pass = false;
  }

  if (!disconnected)
  {
    tft.setCursor(60, 140);

    tft.print("Antenna Connected");
  }

  if (!active_alert)  // If we don't have an active alert and nothing new we do nothing
  {


    // Now we wait for an interrupt and process it

    if (Interrupt_type != 99) // Note: 99 is an impossible type. Since 0 is valid, we use this to test for a real one
    {
      switch (Interrupt_type)
      {

        case 1:
          // It's noise
          int_text = "High Noise";
          tft_event_background = tft_yellow;
          tft.setTextColor(tft_black, tft_yellow);
          break;

        case 2:
          // It's a disturber
          int_text = "Disturber";
          tft_event_background = tft_yellow;
          tft.setTextColor(tft_black, tft_yellow);
          break;

        case 3:
          // It's lightning
          // Activate the Antenna Disconnect relay asap
          int_text = "Lightning";
          tft.setTextColor(tft_yellow, tft_red);
          tft_event_background = tft_red;
          if (!LD_bypassed)
          {
            digitalWrite(LD_relay, HIGH);
            disconnected = true;
          }
          break;
      }
      if (LD_bypassed)
      {
        tft.setTextColor(tft_red, tft_yellow);
        tft_event_background = tft_yellow;
      }

#ifdef debug2
      Serial.println(int_text);
#endif

      clear_text_area(tft_event_background);

      if (Interrupt_type == 3)
      {
        // Add the distance and energy if it's lightning
        tft.setCursor(110, 65);

        if (distance == 1)
        {
          tft.print("Overhead");
        }
        else
        {
          if (distance != 63)
          {
            tft.print(distance); tft.print(" km away");
          } else
          {
            tft.setCursor(80, 65);
            tft.print("Out of Range");
          }
        }
        tft.setCursor(100, 90);

        tft.print("Energy: "); tft.print(map(lightEnergy, 1, 16777216, 1, 100)); tft.print("%");

        if (LD_bypassed)
        {
          tft.setCursor (80, 115);

          tft.print("Bypass Enabled");
        }
        if (disconnected)
        {
          tft.setCursor(40, 140);

          tft.print("Antenna Disconnected");
        }
        tft.setCursor(50, 40);
      }
      else
      {
        tft.setCursor(50, 90);
      }

      tft.print(int_text);
      tft.print(" Detected");
      if (LD_bypassed)
      {
        tft_event_background = tft_yellow;
      }
      else
      {
        if (Interrupt_type == 3)
        {
          tft_event_background = tft_red;
        } else
        {
          tft_event_background = tft_yellow;
        }
      }
      event_time = "Now";
      tick = 0;
      print_timestamp();
      active_alert = true;  // Set the active alert flag
      Interrupt_type = 99; // Reset the interrupt type
      new_event = false;
      last_event = millis();
    }
  }
  else
  {
    // Calculate how long ago and update the event time line
    if (millis() - last_event >= 10000) // Timestamp if not a new event
    {
      timestamp();
    }
  }
}

void count()  // Function to count the number of cycles
{
  freq = freq + 1;
}

// clear_display function
void clear_display()  // Clears the TFT display
{
  tft.fillScreen(tft_black);  // Fill the screen with the background color
}

void poll_interrupt()
{
  bool LD_IRQ = digitalRead(IRQ_pin);
  bool simulated = false;

  if (millis() < hold_down || first_pass)  // Ignore and clear interrupts while held down
  {
    Interrupt_type = 99;
    intVal = lightning.readInterruptReg();
    intVal = 99;
    delay(settle);
    return;
  }

  if (LD_IRQ == HIGH) // We have an interrupt event on the AS3935
  {

    // Read the Interrupt Register to determine the type of interrupt
    intVal = lightning.readInterruptReg();

#ifdef debug2
    Serial.print("Interrupt Received Read Interrupt Value: "); Serial.println(intVal);
#endif

    if (intVal == 0)
    {
      intVal = 99;
      Interrupt_type = 99;
      return;
    }

    /*
      #ifdef debug2

        // --------------------------------------------------------- Simulator block - use some real Disturbers to generate a fake IRQ
        if (intVal == 4)
        {
          // It's a disturber - remap it to something else
          intVal = random(1, 10);
          Serial.print("Int remapped to: "); Serial.println(intVal);
          if (intVal != NOISE_INT && intVal != LIGHTNING_INT && intVal != DISTURBER_INT)
          {
            intVal = 99;
            return;
          }
          else
          {
            if (intVal == 8)  // If the mapped value is lightning
            {
              // Create distance and energy info
              // lightEnergy = number between 1 and 16,777,216
              lightEnergy = random(1, 16777216);
              // distance = number between 1 and 40 but in 15 unique steps - need to use ld_distance[] array
              int x = random(0, 14);
              int ld_distance[] = {1, 5, 6, 8, 10, 12, 14, 17, 20, 24, 27, 31, 34, 37, 40};
              distance = ld_distance[x];
              simulated = true;
            }
          }
        }
        // -----------------------------------------------------

      #endif
    */

    Interrupt_type = 99;
    if (intVal == NOISE_INT)
    {

#ifdef debug2
      Serial.println("Noise Int.");
#endif

      Interrupt_type = 1;
    }
    else if (intVal == DISTURBER_INT)
    {

#ifdef debug2
      Serial.println("Disturber Int.");
#endif

      Interrupt_type = 2;
    }
    else if (intVal == LIGHTNING_INT)  // Interrupt Value 0 is a distance update due to stats purge, handle like a new event
    {

#ifdef debug2
      Serial.println("Lightning Detect");
#endif

      Interrupt_type = 3;
      // Lightning! Now how far away is it? Distance estimation takes into
      // account previously seen events.
      if (!simulated)
      {
        distance = lightning.distanceToStorm();
      }

#ifdef debug2
      if (distance != 63)
      {
        Serial.print("Approx: ");
        Serial.print(distance);
        Serial.println("km away!");
      } else
      {
        Serial.println("Out of Range ");
      }
#endif

      // "Lightning Energy" is strictly a relative number
      // that does not have any meteorological significance.
      if (!simulated)
      {
        lightEnergy = lightning.lightningEnergy();
      }

#ifdef debug2
      Serial.print("Energy: ");
      Serial.println(lightEnergy);
#endif

    }

#ifdef debug2
    Serial.print("Interrupt type = "); Serial.println(Interrupt_type);
#endif
    active_alert = false; // Reset the alert so we decode the new one
  }
}

void timestamp()
{
  // Displays the elapsed time of an event
  // Uses the return statement to avoid a pile of messy nested if statements

  if ((millis() - last_scan) >= 10000) // Run every 10 seconds
  {
    // Calculates and prints how long ago the last event was

    if (auto_clear && tick > clear_alert_time)  // if autoclear is enabled and we've exceeded the time
    {
      digitalWrite(LD_relay, LOW);
      disconnected = false;
      tft_event_background = tft_green;
      clear_text_area(tft_event_background);
      tft.setTextColor(tft_black, tft_event_background);
      tft.setCursor(35, 70);
      tft.print("No Lightning Detected");
      tft.setCursor(60, 140);
      tft.print("Antenna Connected");
      active_alert = false;
      return;
    }
    if (millis() < last_event)
    {
      // We've rolled over - reset last event to current time and clear display)
      last_event = millis();
      last_scan = last_event;
      active_alert = false;
      return;
    }
    if (millis() - last_event >= 10000) // Skip the first tick after an event
    {
      // 10 second tick
      tick = tick + 10;
    }
    //    long time_diff = ((millis() / 1000) - (last_event / 1000)); // Difference in seconds

#ifdef debug1
    Serial.print("Tick: "); Serial.print(tick); Serial.print(" ***");
#endif

    last_scan = millis();
    if (tick < 60)
    {
      // It's less than a minute old
      event_time = String(tick) + " Second";
      text_spacing = 0;
      if (tick > 1)
      {
        event_time = event_time + "s";
      }
      event_time = " " + event_time + " Ago";
      print_timestamp();
      return;
    }
    if ((tick >= 60) && (tick < 3600))  // It's over a minute and less than an hour - set timestamp to minutes
    {
      if (int(tick / 60) != tick / 60) // skip timestamp if not a full minute
      {

#ifdef debug1
        Serial.println(" 10 second tick - No update");
#endif

        return;
      }
      int minutes = tick / 60;
      event_time = String(minutes) + " Minute";
      text_spacing = -10;
      if (minutes > 1)
      {
        event_time = event_time + "s";
      }
      event_time = " " + event_time + " Ago";
      print_timestamp();
      return;
    }

    int hours = tick / 3600; // Is it over an hour and less than a day?
    if ((hours >= 1 && hours < 24) && (tick / 3600 == hours))
    {
      if (hours < 24)
      {
        event_time = String(hours) + " Hour";
        if (hours > 1)
        {
          event_time = event_time + "s";
        }
      }
      event_time = " " + event_time + " Ago";
      text_spacing = 20;
      print_timestamp();
      return;
    }
    else
    {
      // If we get here, we're over 24 hours
      int days = tick / 86400;
      if (days != (tick / 86400))
      {

#ifdef debug1
        Serial.println(" 10 second tick - No update");
#endif

        return;
      }
      event_time = String(days) + " Day";
      if (days > 1)
      {
        event_time = event_time + "s";
      }
      event_time = " " + event_time + " Ago";
      text_spacing = 15;
      print_timestamp();
      return;
    }
  }
}

void print_timestamp()
{

#if defined(debug1) || defined(debug2)
  Serial.println(event_time);
#endif

  // Erase the timestamp area on the TFT
  tft.fillRect(0, 165, 320, 16, tft_event_background);
  if (event_time == "Now")
  {
    tft.setCursor (130, 165);
    last_event = millis();
  }
  else
  {
    tft.setCursor (70 + text_spacing, 165);
  }
  tft.print(event_time);
}

void update_status()
{
  tft.setTextSize(1);

  // Draw lines for config/status area
  tft.drawFastHLine(0, 185, 320, tft_cyan);
  tft.drawFastVLine(140, 185, 45, tft_cyan);

#ifdef debug
  Serial.print("Tuning Cap: "); Serial.print(tuning_cap); Serial.println(" pf");
#endif

  tft.setCursor(5, 190);
  tft.print("Tuning Cap: ");
  tft.print(tuning_cap); tft.print(" pF");
  tft.setCursor(180, 190);
  tft.print("Detector: Online");

  // Set Indoor/Outdoor Mode and display
  lightning.setIndoorOutdoor(LD_mode);

  int enviVal = lightning.readIndoorOutdoor();

#ifdef debug
  Serial.print("Mode: ");
#endif

  tft.setCursor (5, 200);
  tft.print("Mode: ");
  if (enviVal == INDOOR )
  {
    
#ifdef debug
    Serial.println("Indoor");
#endif

    tft.print("Indoor");
  }
  else
  {
    if (enviVal == OUTDOOR )
    {
#ifdef debug
      Serial.println("Outdoor");
#endif
      tft.print("Outdoor");
    }
    else
    {
#ifdef debug
      Serial.println(enviVal, BIN);
#endif
      tft.print("Unknown");
    }
  }

  // Enable or Disable Disturbers and Display
  lightning.maskDisturber(LD_Disturber);
  tft.setCursor (180, 200);
  tft.print("Disturbers: ");
  int maskVal = lightning.readMaskDisturber();

#ifdef debug
  Serial.print("Disturbers: ");
#endif

  if (maskVal == 1)
  {
    
#ifdef debug
    Serial.println("Disabled");
#endif

    tft.print("Disabled");
  }
  else
  {
    
#ifdef debug
    Serial.println("Enabled");
#endif

    tft.print("Enabled");
  }

  // Set/Display Noise Floor Level
  lightning.setNoiseLevel(noiseFloor);
  int noiseVal = lightning.readNoiseLevel();

#ifdef debug
  Serial.print("Noise Level: ");
  Serial.println(noiseVal);
#endif

  tft.setCursor (5, 210);
  tft.print("Noise Level: ");
  tft.print(noiseVal);

  // Set/Display Watchdog Threshold
  lightning.watchdogThreshold(watchDogVal);
  int watchVal = lightning.readWatchdogThreshold();

#ifdef debug
  Serial.print("Watchdog: ");
  Serial.println(watchVal);
#endif

  tft.setCursor (180, 210);
  tft.print("Watchdog Threshold: ");
  tft.print(watchVal);

  // Set/Display Spike rejection Level
  lightning.spikeRejection(spike);
  int spikeVal = lightning.readSpikeRejection();

#ifdef debug
  Serial.print("Spike: ");
  Serial.println(spikeVal);
#endif

  tft.setCursor (5, 220);
  tft.print("Spike Rejection: ");
  tft.print(spikeVal);

  // Set/Display Lightning Threshold

  lightning.lightningThreshold(lightningThresh);
  uint8_t lightVal = lightning.readLightningThreshold();

#ifdef debug
  Serial.print("# of strikes req: ");
  Serial.println(lightVal);
#endif

  tft.setCursor (180, 220);
  tft.print("Lightning Threshold: ");
  tft.print(lightVal);
}

void start_tft()  // Starts the TFT display and shows the startup screen
{
  tft.begin();  // Start the TFT display

#ifdef debug

  Serial.println("TFT Start");  // Verifies that the TFT is operational
  Serial.print("Display Pwr Mode: 0x");
  Serial.println(tft.readcommand8(ILI9341_RDMODE), HEX);
#endif

  clear_display();  // Clear the TFT display
  tft.setTextColor(tft_cyan, tft_black);  // Set the display text to Cyan. Black background color required to "clear" previous text
  tft.setRotation(3); // horizontal display // change rotation as needed/desired
  tft.setTextSize(4);  // large font
  tft.setCursor(100, 30);  //Set the Cursor and display the startup screen
  tft.print("KW5GP");
  tft.setCursor(55, 100);
  tft.print("Lightning");
  tft.setCursor(65, 160);
  tft.print("Detector");
  delay(5000);  //Wait 5 seconds then clear the startup message
  clear_display();  // Clear the TFT display
  tft.setTextSize(2);
}

void check_LD_module() // Function to do basic lightning detector module tests
{
  if (!lightning.beginSPI(spiCS))// This function/test really appears to not work but is left in just in case
  {

#ifdef debug
    Serial.println ("Lightning Detector did not start, freezing!");
#endif

    tft.setTextColor(tft_red, tft_black);  // Set the display text to Red
    tft.setCursor (50, 50);
    tft.print("Lightning Detector");
    tft.setCursor (75, 90);
    tft.print("Startup Failed");
    tft.setCursor (35, 150);
    tft.print("Check Wiring / Module");
    while (1);  // Loop here forever and not proceed
  }

  // We'll put our own test here. We'll set the Tuning cap to 0pF, then 96pF
  // If we don't read 96pF, then we most likely aren't communicating with the module
  lightning.tuneCap(0);
  lightning.tuneCap(96);
  if (lightning.readTuneCap() != 96) // Check to see if we can read what we wrote
  {
    // We can assume the operation failed
#ifdef debug
    Serial.println("Lightning Detector is not communicating");
    Serial.println("Check wiring / verify module is good");
#endif

    tft.setTextColor(tft_red, tft_black);  // Set the display text to Red
    tft.setCursor (10, 50);
    tft.print("Lightning Detector Module");
    tft.setCursor (35, 90);
    tft.print("Communication Failure");
    tft.setCursor (35, 150);
    tft.print("Check Wiring / Module");
    while (1);  // Loop here forever and not proceed
  }

  // If we get here, we assume the AS3935 Module is functioning
  lightning.tuneCap(0); // Set the tuning cap back to 0
  lightning.changeDivRatio(16); // Set the division ratio just to make sure
  // Read the division ratio - 16 is default.
  byte divVal = lightning.readDivRatio();

#ifdef debug
  Serial.print("Division: "); Serial.println(divVal);
#endif

}

void check_calibration()
{
  // Read the Tuning Jumper to determine if we use stored cap value or calculate it
  if (!digitalRead(LD_Tune_jumper))
  {
    // The Tuning jumper is on, run calibration

#ifdef debug
    Serial.println("Ready to tune");
#endif

    // The frequency of the antenna is divided by the "division ratio" which is
    // set to 16 by default. This can be changed to 32, 64, or 128 using the
    // function call below. As an example when reading the frequency of a
    // new board, the frequency 31.04kHz. Multiplying that frequency by the
    // division ratio gives 496.64kHz, which is less than 1 percent within the
    // OPTIMAL range of 3.5 percent specified in the datasheet.

#ifdef debug
    Serial.println("**** Beginning Calibration ****");
#endif

    tft.setCursor (90, 70);
    tft.print("Calibrating");
    tft.setCursor (115, 100);
    tft.print("Antenna");
    tft.setCursor (60, 130);
    tft.print("Tuning Capacitor");
    tft.setCursor (90, 170);
    tft.print("Please Wait");
    // Set the best results variables to a failing value
    best_cap = 9999;
    best_diff = 9999;
    for (int i = 0; i <= 120; i = i + 8) // Tuning cap value in pF
    {
      lightning.tuneCap(i);
      tuneVal = lightning.readTuneCap();

#ifdef debug
      Serial.print("Tuning Cap: ");
      Serial.print(tuneVal);
#endif

      // set up ISR - Calls count() whenever pin changes (twice per cycle)
      attachInterrupt(digitalPinToInterrupt(IRQ_pin), count, CHANGE);
      end_time = micros() + 500000;
      // Turn on the oscillator
      lightning.displayOscillator(true, ANTFREQ);
      while (micros() < end_time)
      {
        // Do nothing - count is incremented by Interrupt Service Routine
      }
      // Turn off the oscillator
      lightning.displayOscillator(false, ANTFREQ);
      freq = freq * 2;
      if (freq == 0)
      {
        // We have an oscillator failure

#ifdef debug
        Serial.println(); Serial.println("*** Oscillator Failure - Aborting ****");
#endif

        clear_display();
        tft.setTextColor(tft_red, tft_black);  // Set the display text to Red
        tft.setCursor (10, 50);
        tft.print("Lightning Detector Module");
        tft.setCursor (50, 90);
        tft.print("Oscillator Failure");
        tft.setCursor (35, 150);
        tft.print("Check Wiring / Module");
        while (1);  // Loop here forever and not proceed
        osc_fail = true;
        i = 999;
      } else
      {
        // At this point, freq should have the value of the frequency * 2
#ifdef debug
        Serial.print("pF    Count = "); Serial.print(freq); Serial.print("    Freq: "); Serial.print(freq / 2);
        Serial.print("    Resonance: "); Serial.print(freq * 8);
#endif

        diff = (500000 - (freq * 8)) / 5000;

#ifdef debug
        Serial.print("   Diff: "); Serial.print(diff, 2); Serial.println(" % ");
#endif

        if (abs(diff) < abs(best_diff) && freq != 0)
        {
          best_diff = diff;
          best_cap = i;
        }
        freq = 0;
        end_time = 0;
      }
    }
    if (!osc_fail)
    {

#ifdef debug
      Serial.print("Best Cap: "); Serial.print(best_cap); Serial.print("pf    Best Diff: "); Serial.print(best_diff, 2);
      Serial.println(" % ");
#endif

      if (abs(best_diff) <= 3.5)
      {
        // The module passes the calibration test

#ifdef debug
        Serial.println("**** Antenna Tuning Successful ****");
#endif

      } else
      {

#ifdef debug
        Serial.println("**** Unable to tune to acceptable tolerance ****");
#endif

        clear_display();
        tft.setTextColor(tft_red, tft_black);  // Set the display text to Red
        tft.setCursor (10, 50);
        tft.print("Lightning Detector Module");
        tft.setCursor (30, 110);
        tft.print("Unable to Tune Antenna");
        tft.setCursor (25, 140);
        tft.print("To Acceptable Tolerance");
        tft.setCursor (90, 190);
        tft.print("Check Module");
        while (1);  // Loop here forever and not proceed
        // Hang until reset/fixed
        while (true)
        {
        }
      }
      tuning_cap = best_cap;
    }
    else
    {
      // Hang until reset/fixed
      while (true)
      {
      }
    }
    // Turn off the calibration Interrupt Service Routine (ISR)
    detachInterrupt(digitalPinToInterrupt(IRQ_pin));
  }
  else
  {
    // Just leave at the stored value
  }
  lightning.tuneCap(tuning_cap); // Set the tuning capacitor
  // Read the internal capacitor value, it will return the value in pF.

#ifdef debug
  Serial.print("Internal Cap set to: ");
  Serial.print(lightning.readTuneCap()); Serial.println(" pF");
#endif

  // Now calibrate the internal oscillators of the IC - now that the
  // resonance frequency of the antenna is tightly trimmed. They are calibrated
  // off of the antenna frequency.
  if (lightning.calibrateOsc())
  {
#ifdef debug
    Serial.println("Onboard Oscillators Calibrated");
#endif
  }
  else
  {
#ifdef debug
    Serial.println("Onboard Oscillators Failed Calibration");
#endif
    clear_display();
    tft.setTextColor(tft_red, tft_black);  // Set the display text to Red
    tft.setCursor (10, 50);
    tft.print("Lightning Detector Module");
    tft.setCursor (50, 100);
    tft.print("Unable to Calibrate");
    tft.setCursor (50, 130);
    tft.print("Onboard Oscillators");
    tft.setCursor (90, 190);
    tft.print("Check Module");
    // Hang until reset/fixed
    while (true)
    {
    }
  }
  clear_display();
  tft.setCursor (10, 50);
  tft.print("Lightning Detector Module");
  tft.setCursor (40, 120);
  tft.print("Calibration Complete");
  delay(3000);
}

void clear_text_area(int text_background)
{
  // Fill the text area with the appropriate background color Red = active alert, Yellow = alert over an hour old, Green = no alerts
  tft.fillRect(0, 0, 320, 185, text_background); //erase starting at x,y for length and width
}


void simulate_interrupt()
{

#ifdef debug2
  long curr_time = millis();
  int ld_distance[] = {1, 5, 6, 8, 10, 12, 14, 17, 20, 24, 27, 31, 34, 37, 40};
  if (curr_time - test_tick > 10000) // 10 second timer
  {
    Serial.print("10 second test tick"); Serial.print(" test_tick : "); Serial.println(test_tick);
    test_tick = curr_time;
    if (((curr_time / 1000) - (last_sim / 1000)) >= trigger_time) // Run every 3 minutes
    {
      Interrupt_type = random(1, 4);
      Interrupt_type = 3;   // ---------------------------------------------test
      new_event = true;
      active_alert = false; // Clear any previous event
      Serial.print("Random Interrupt: ");
      Serial.print(Interrupt_type);
      Serial.print("   ");
      last_sim = millis();
      Serial.print("  Millis(): "); Serial.print(millis()); Serial.print("   last_sim:"); Serial.println(last_sim);
      trigger_time = 180;
      if (Interrupt_type == 3)
      {
        // Lightning simulation - needs additional variables filled out
        // lightEnergy = number between 1 and 16,777,216
        lightEnergy = random(1, 16777216);
        // distance = number between 1 and 40 but in 15 unique steps - need to use ld_distance[] array
        int x = random(0, 14);
        distance = ld_distance[x];
        Serial.print("Energy: "); Serial.print(lightEnergy); Serial.print(" Distance: "); Serial.println(distance);
      }
    }
  }
#endif
}
