/* CW Decoder

  Modified and added TFT display by Glen Popiel - KW5GP

  Uses MORSE ENDECODER Library by raronzen
  Copyright (C) 2010, 2012 raron
  GNU GPLv3 license (http://www.gnu.org/licenses)
  Contact: raronzen@gmail.com  (not checked too often..)
  Details: http://raronoff.wordpress.com/2010/12/16/morse-endecoder/
*/

//#define debug
//#define debug1  // Used to simulate received CW characters for testing

#include <avr/pgmspace.h>
#include <MorseEnDecoder.h>  // Morse EnDecoder Library
#include <TFT_ILI9341.h> // Hardware-specific library
#include <SPI.h>

#define morseInPin 2
#define Speed_Pin A0
#define BAUD_RATE 19200

#define TFT_CS     10  // Assign the TFT CS to pin 10
#define TFT_DC     9  // Assign the TFT DC to pin 9

#define cw_font_size 3  // The RTTY text display font size
#define status_font_size 2  // The font size for the status line

#define tft_cyan ILI9341_CYAN // Simplifies the color definitions for the TFT
#define tft_black ILI9341_BLACK
#define tft_red ILI9341_RED
#define tft_green ILI9341_GREEN
#define tft_yellow ILI9341_YELLOW

#ifdef debug1
char Code[41];  // The array to hold the CW Alphabet 0-9,-./A-Z - Used for testing
#endif

int background = tft_black; // Set the TFT background and text color
int text_color = tft_cyan;

String text = "";
int read_speed;  // Variable for desired CW speed setting
int current_speed = -1; // variables to track speed pot
int index; // variable to select CW character to send
String cw_data = "";  // String variable to hold cw text data
char cw_rx = "";  // Single character string variable to hold CW characterreceived

// Use hardware SPI for TFT
TFT_ILI9341 tft = TFT_ILI9341();       // Invoke custom library

morseDecoder morseInput(morseInPin, MORSE_KEYER, MORSE_ACTIVE_LOW);  // Define the Morse objects

void setup()
{
  pinMode(morseInPin, INPUT);

#ifdef debug1
  populate_character_array();  // Populate the character array if simulating receive
#endif

  start_tft();  // Start the TFT display

  //#ifdef debug
  Serial.begin(BAUD_RATE); // initialize serial communication on the USB port
  //#endif

}  // End Setup Loop

void loop()
{
  read_speed = map(analogRead(Speed_Pin), 10, 1000, 5, 35); // Read the potentiometer to determine code speed

  if (current_speed != read_speed)  // If the Speed Pot has changed, update the speed and LCD
  {
    current_speed = read_speed;  // Set the current speed to the desired speed
    morseInput.setspeed(read_speed);  // Call the set speed function
    text = String(current_speed) + " wpm";  // set up the LCD display with the current speed

#ifdef debug
    Serial.print("CW speed set to: ");
    Serial.println(read_speed);
#endif

    update_status();
    // Put the cursor back
    tft.setTextSize(cw_font_size);  // Set the tft text font size
    tft.setCursor(0, 5); // Move the cursor to the top line

    tft.print(cw_data); // Print the tft data string variable
  }

#ifdef debug1
  simulate_character(); // Simulate receiving a CW character
#ifdef debug
  Serial.print("RX Index: "); Serial.print(index); Serial.print(" Char: "); Serial.println(cw_rx);
#endif

  Serial.print(cw_rx)
  update_text();
  
#else // Do this if no debug1 flag
  morseInput.decode();  // Decode incoming CW
  if (morseInput.available())  // If there is a character available
  {
    cw_rx = morseInput.read();  // Read the CW character
    
#ifdef debug
    Serial.print("Character Received: "); Serial.print(cw_rx); Serial.print("  ASCII: "); Serial.println(int(cw_rx));
#endif
    Serial.print(cw_rx);
    update_text();
  }
#endif

}  // End Main Loop

void start_tft()  // Starts the TFT display and shows the startup screen
{
  tft.begin();  // Start the TFT display

#ifdef debug1
  Serial.println("TFT Start");  // Verifies that the TFT is operational
  Serial.print("Display Pwr Mode: 0x");
  Serial.println(tft.readcommand8(ILI9341_RDMODE), HEX);
#endif

  clear_display();  // Clear the TFT display
  tft.setTextColor(text_color, background);  // Set the display text to black. Green background color required to "clear" previous text
  tft.setRotation(3); // horizontal display // change rotation as needed/desired
  tft.setTextSize(cw_font_size +1);  // large font
  tft.setCursor(90, 30);  //Set the Cursor and display the startup screen
  tft.print("KW5GP");
  tft.setCursor(130, 90);
  tft.print("CW");
  tft.setCursor(70, 150);
  tft.print("Decoder");

  delay(5000);  //Wait 5 seconds then clear the startup message
  clear_display();  // Clear the TFT display

  // Draw lines for config/status area
  tft.drawFastHLine(0, 210, 320, tft_cyan);

  clear_text_area(background);
}

void clear_display()  // Function to clear the TFT display
{
  tft.fillScreen(background); // Clear the displayusing current background color
}

void update_status() // Prints status messages on bottom line of TFT display
{
  tft.setTextSize(status_font_size);  // Set the font size to the status line font size
  tft.setCursor(80, 220);  // Move to the end of the status line
  tft.print("CW Speed: ");
  tft.print(current_speed);  // Display the baud rate
  tft.print(" ");

  tft.setTextSize(cw_font_size);  // Reset the text size for the rtty text area
  tft.setCursor(0, 5);  // Move to the top line of the display
}

void clear_text_area(int text_background)
{
  // Fill the text area with the appropriate background color Red = active alert, Yellow = alert over an hour old, Green = no alerts
  tft.fillRect(0, 0, 320, 200, text_background ); //erase starting at x,y for length and width
}

#ifdef debug1
void populate_character_array() // Used to simulate received characters during testing
{
  // populate the array containing the characters to use (0-9 , . / ? A-Z)
  for (int x = 0 ; x < 14 ; x++) // 0-9,-.
  {
    Code[x] = char(44 + x);
  }

  for (int x = 14 ; x < 40 ; x++) // A-Z
  {
    Code[x] = char(51 + x);
  }

  Code[40] = char(63);  // add ? character

}

void simulate_character()
{
  index = int(random(41));  // Randomly pick a character from the character array
  cw_rx = Code[index];  // Assign the value of the selected character to c
  delay(1000);
}

#endif

void update_text() // Updates the tft text on the display
{
  tft.setTextSize(cw_font_size);  // Set the tft text font size
  if (cw_data.length() >= 39)  // Scroll bottom line up a line if at end of line 6
  {
    Serial.println("");
    cw_data = cw_data.substring(17);  // Remove the first 13 characters (the top line)
    tft.setCursor(0, 5); // Move the cursor to the top line
    tft.fillRect(0, 0, 319, 99, tft_black); // Clear the tft data area
    tft.print(cw_data); // Print the tft data string variable
    tft.setCursor(0, 69);  // Set the cursor to the start of line 
  }

  cw_data = cw_data + cw_rx;  // Add the character to the tft data string
  tft.print(cw_rx); // display the received character

#ifdef debug
  if (cw_data.length() != 0)
  {
    Serial.print("cw_data string length: ");
    Serial.print(cw_data.length());
    Serial.println(" ");
  }
#endif

}
