/*
  DDS VFO - Tunes from 1 to 30 MHz with programmable step speed
  Author: Glen Popiel - KW5GP

  Uses M0XPD DDS Library by Paul Darlington, M0XPD
*/

#include <DDS.h>  // include the M0XPD DDS Library
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library
#include <SPI.h>  // SPI library so we can communicate with the TFT display
#include <Encoder.h>  // Include the Rotary Encoder Library

//======================================
// AD9850 Module....
// set pin numbers:
const int W_CLK = 8;
const int FQ_UD = 10;
const int DATA = 4;
const int RESET = 9;

#define TFT_CS     5  // Assign the TFT CS to pin 10
#define TFT_RST    7  // Assign the TFT RST to pin 7
#define TFT_DC     6  // Assign the TFT DC to pin 6

#define tft_delay 10  // set the TFT command delay to 10ms

const int encoder_PinA = 3;  // Pin A of the Rotary Encoder
const int encoder_PinB = 2;  // Pin B of the Rotary Encoder
const int encoder_Switch_Pin = A4;  // Encoder Pushbutton Switch input pin

double step_value[] = {1, 10, 100, 1000, 10000, 100000, 1000000};

const int debounceInterval = 500;  //disable step change for 500 ms between encoder pushbutton press
boolean encoder_Switch;
int current_step = 0;  // Default Tuning Step is 1 Hz
int encoder_Pos = 0;
int old_encoder_Pos = 0;

const double range_low = 1000000; // Low end of range (1 MHz)
const double range_high = 30000000; // High end of range (30 MHz)
unsigned long int delay_time = 2000;

double start_freq = 7000000;  // Starting freq

double dds_freq = start_freq;

boolean first_pass = true;  // Variable used to determine if first pass through the main loop

// Instantiate the DDS
DDS dds(W_CLK, FQ_UD, DATA, RESET);

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST);  // Initialize the TFT display instance

extern uint8_t SmallFont[];  // define the Nokia Font

// Instantiate the Rotary Encoder
Encoder Enc(encoder_PinA, encoder_PinB);

void setup()
{
  tft.initR(INITR_18BLACKTAB);   // initialize a 1.8" TFT with ST7735S chip, black tab
  delay(tft_delay);
  tft.fillScreen(ST7735_BLACK); // Clear the display - fill with BLACK background
  delay(tft_delay);
  tft.setRotation(1); // Set the screen rotation
  delay(tft_delay);
  tft.setTextWrap(false); // Turn off Text Wrap
  delay(tft_delay);
  tft.setTextSize(3); // Set the Font Size
  delay(tft_delay);
  tft.setTextColor(ST7735_BLUE); //Set the Text Color
  delay(tft_delay);
  tft.setCursor(40, 20);  //Set the Cursor and display the startup screen
  delay(tft_delay);
  tft.print("KW5GP");
  delay(tft_delay);
  tft.setTextSize(2);
  delay(tft_delay);
  tft.setCursor(35, 60);
  delay(tft_delay);
  tft.print("1-30 MHz");
  delay(tft_delay);
  tft.setCursor(40, 90);
  delay(tft_delay);
  tft.print("DDS VFO");
  delay(tft_delay);

  // start up the DDS...
  dds.init();
  // (Optional) trim if your xtal is not at 125MHz...
  dds.trim(125000322); // enter actual osc freq

  delay(3000);
  tft.fillScreen(ST7735_BLACK); // Clear the display
  delay(tft_delay);

  update_display(); // update the display
  first_pass = false; // turn off the update display first pass flag

  // Set up the Rotary Encoder
  pinMode (encoder_PinA, INPUT);
  pinMode (encoder_PinB, INPUT);
  pinMode (encoder_Switch_Pin, INPUT);
  // Enable the Internal Pull-up resistor on the Encoder Inputs
  digitalWrite(encoder_PinA, HIGH);
  digitalWrite(encoder_PinB, HIGH);
  digitalWrite(encoder_Switch_Pin, HIGH);

  // start the oscillator...
  set_DDS(dds_freq);

  update_display();

}  // End Setup Loop

void loop()
{

  read_encoder(); // Read the encoder and set the Tx/Rx frequencies
  read_pushbutton();  // Read the Encoder Pushbutton switch and set Fast/Slow tuning mode

}  // End Main Loop

void read_encoder()
{
  // Read the Encoder
  encoder_Pos = Enc.read() / 4; // divide by 4 to match encoder detent
  if (encoder_Pos != old_encoder_Pos) // If the Encoder has changed update freq
  {
    if (encoder_Pos < old_encoder_Pos) // If we're increasing frequency
    {
      if (dds_freq >= range_high) // Limit to top end of band
      {
        dds_freq = range_high;
      } else {
        dds_freq = dds_freq + step_value[current_step];
      }

    } else {
      if (dds_freq <= range_low)// We're decreasing frequency, limit to low end of band
      {
        dds_freq = range_low;
      } else {
        dds_freq = dds_freq - step_value[current_step];
      }
    }

    //    dds.setFrequency(tx_freq); // Set the TX frequency
    old_encoder_Pos = encoder_Pos;
    set_DDS(dds_freq);
    update_display();

  }
}

void read_pushbutton()
{
  // Read the Encoder Pushbutton Switch
  encoder_Switch = digitalRead(encoder_Switch_Pin);
  if (encoder_Switch == LOW && millis() > delay_time) // Check to see if pressed
  {
    // if it's changed, toggle the step size but don't allow again for debounce time
    delay_time = millis() + debounceInterval; // if it's changed, toggle the step size but don't allow again for debounce time
    current_step = current_step + 1; // Change the Step pointer
    if (current_step > 6)
    {
      current_step = 0;
    }

    update_display();
  }
}

void set_DDS(double set_freq)
{
  dds.setFrequency(set_freq);  // Set the DDS to the starting freq
  if (set_freq >= range_high)
  {
    set_freq = range_high;
  }
  if (set_freq <= range_low)
  {
    set_freq = range_low;
  }

}

void update_display() // Updates the TFT display
{
  if (first_pass) // Only do this part the first time the function is called
  {
    // Clear the screen and display normal operation
    tft.fillScreen(ST7735_BLACK); // Clear the display
    delay(tft_delay);
    tft.setTextSize(1); // Set the text size to 1
    delay(tft_delay);
    tft.setTextColor(ST7735_BLUE); // Set the text color to BLUE
    delay(tft_delay);
    tft.setCursor(40, 5);
    delay(tft_delay);
    tft.print("1-30MHz DDS VFO");  // Display screen title
    tft.setCursor(60, 40);
    delay(tft_delay);
    tft.print("Frequency");  // Display Frequency

  } else
  {
    clear_data();  // Clear the value display area
    tft.setTextSize(2); // Set the text size to 2
    delay(tft_delay);
    tft.setCursor(45, 50);
    delay(tft_delay);
    tft.print(dds_freq,0); // Display DDS Frequency Value
    tft.setCursor(40, 90);
    delay(tft_delay);
    tft.setTextSize(1); // Set the text size to 1
    delay(tft_delay);
    tft.print("Step: ");  // Display Step Value
    tft.setCursor(70, 90);
    delay(tft_delay);
    tft.print(step_value[current_step],0); // Display Frequency Step Value
  }
}

void clear_data() //Clears the data area of the display
{
  tft.fillRect(0, 50, 150, 60, ST7735_BLACK); // Clear the CAT data area
  delay(tft_delay);
}
