/*
  Random Code Practice Oscillator by Glen Popiel - KW5GP

  Uses Adafruit ILI9341 and GFX TFT display libraries
  Uses Morse Library to send CW tones
  Uses SPI library to communicate with SPI TFT display

*/

//#define debug   // Enables diagnostic information

#include <SPI.h>  // SPI library required for SPI devices
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ILI9341.h>
#include <Morse.h>   // Use the Morse Library

#define TFT_CS     10  // Assign the TFT CS to pin 10
#define TFT_DC     7  // Assign the TFT DC to pin 7

#define baud_rate 115200
#define tft_cyan ILI9341_CYAN
#define tft_black ILI9341_BLACK
#define tft_red ILI9341_RED
#define tft_green ILI9341_GREEN
#define tft_yellow ILI9341_YELLOW

#define tft_font_size 4  // The TFT text display font size
#define status_font_size 2  // The font size for the status line

//char c;  // The CW character to send
char Code[41];  // The array to hold the CW Alphabet 0-9,-./A-Z
int key_speed = 10 ;  // CW Speed
int index; // variable to select CW character to send
const int beep_pin = 6; // Pin for CW tone
int prev_key_speed ;
String tft_data = "";  // String variable to hold tft text data
char tft_char = "";  // Single character string variable to hold CW character to send

// Use hardware SPI (on Uno/Nano, #13, #12, #11) and the above for CS/DC
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);  // Create the TFT instance

Morse morse(beep_pin, key_speed, 1);  // Set the Code Library to Beep on Pin 6 at the selected Key Speed

void setup()
{
#ifdef debug
  Serial.begin(baud_rate);
  Serial.println("Starting Serial Port for debugging");
#endif

  start_tft();

  // populate the array containing the characters to use (0-9 , . / ? A-Z)
  for (int x = 0 ; x < 14 ; x++) // 0-9,-.
  {
    Code[x] = char(44 + x);
  }

  for (int x = 14 ; x < 40 ; x++) // A-Z
  {
    Code[x] = char(51 + x);
  }

  Code[40] = char(63);  // add ? character

  // randomize
  randomSeed(analogRead(0)); // Seed the Random Number Generator
  delay(1000);
}

void loop()
{

  key_speed = map(analogRead(0), 10, 1010, 5, 35); // Read the potentiometer to determine code speed
  if (key_speed > 35)
  {
    key_speed = 35;
  }
  if (key_speed < 5)
  {
    key_speed = 5;
  }
  if (key_speed != prev_key_speed)
  {
    Morse morse(beep_pin, key_speed, 1);  // Set the Code Library to Beep at the selected Key Speed
    prev_key_speed = key_speed;
    update_status();
    // Put the cursor back
    tft.setTextSize(tft_font_size);  // Set the tft text font size
    tft.setCursor(0, 5); // Move the cursor to the top line

    tft.print(tft_data); // Print the tft data string variable
#ifdef debug
    Serial.print("Key Speed = "); Serial.println(key_speed);
#endif
  }

  index = int(random(41));  // Randomly pick a character from the character array
  tft_char = Code[index];  // Assign the value of the selected character to c

#ifdef debug
  Serial.print("Char = "); Serial.println(tft_char);
#endif

  morse.send(tft_char);  // Send the character in CW
  update_text();
}

void clear_display()  // Function to clear the TFT display
{
  tft.fillScreen(tft_black); // Clear the display - set to use a black background
}

void update_status()
{
  // Draw lines for config/status area
  tft.drawFastHLine(0, 210, 320, tft_cyan);
  tft.setTextSize(status_font_size);  // status line font
  tft.setCursor (70, 220);
  tft.print("Speed: ");
  tft.print(key_speed); tft.print(" wpm   ");
}

void update_text() // Updates the tft text on the display
{
  tft.setTextSize(tft_font_size);  // Set the tft text font size
  if (tft_data.length() >= 78)  // Scroll bottom line up a line if at end of line 6
  {
    tft_data = tft_data.substring(13);  // Remove the first 13 characters (the top line)
    tft.setCursor(0, 5); // Move the cursor to the top line
    tft.fillRect(0, 0, 319, 209, tft_black); // Clear the tft data area
    tft.print(tft_data); // Print the tft data string variable
    tft.setCursor(0, 165);  // Set the cursor to the start of line 6
  }

  tft_data = tft_data + tft_char;  // Add the character to the tft data string
  tft.print(tft_char); // display the received character

#ifdef debug
  if (tft_data.length() != 0)
  {
    Serial.print("tft_data string length: ");
    Serial.print(tft_data.length());
    Serial.print(" ");
  }
#endif

}

void clear_text() // Clears the text area of the TFT
{
  tft.setCursor(0, 5); // Move the cursor to the top line
  tft.fillRect(0, 0, 319, 180, tft_black); // Clear the tft data area
  tft_data = "";
}

void start_tft()  // Starts the TFT display and shows the startup screen
{
  tft.begin();  // Start the TFT display

#ifdef debug

  Serial.println("TFT Start");  // Verifies that the TFT is operational
  Serial.print("Display Pwr Mode: 0x");
  Serial.println(tft.readcommand8(ILI9341_RDMODE), HEX);
#endif

  clear_display();  // Clear the TFT display
  tft.setTextColor(tft_cyan, tft_black);  // Set the display text to Cyan. Black background color required to "clear" previous text
  tft.setRotation(3); // horizontal display // change rotation as needed/desired
  tft.setTextSize(4);  // large font
  tft.setCursor(95, 40);  //Set the Cursor and display the startup screen
  tft.print("KW5GP");
  tft.setCursor(30, 100);
  tft.print("Random Code");
  tft.setCursor(60, 140);
  tft.print("Practice");
  tft.setCursor(50, 180);
  tft.print("Generator");
  delay(5000);  //Wait 5 seconds then clear the startup message
  clear_display();  // Clear the TFT display
}
