/*
  Talking UTC Clock/GPS/Grid Square Display by Glen Popiel - KW5GP

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 3 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

  ILI9341 TFT module is connected to the following pins.
      CLK  - Pin 12
      MOSI  - Pin 11
      CS   - Pin 10
      DC   - Pin 7
      RST - connected to Vcc pin
      LED - connected to Vcc pin
*/

//#define debug // Enables debug mode

#include "DFRobot_SpeechSynthesis_V2.h"
DFRobot_SpeechSynthesis_I2C ss;

#include <SPI.h>
#include <Adafruit_GFX.h> // The Adafruit Graphics Library
#include <Adafruit_ILI9341.h> // The ILI9341 TFT Display Library
#include <SoftwareSerial.h>  // include the SoftwareSerial library so we can use it to talk to GPS
#include <TinyGPSPlus.h>  // The TinyGPSPlus GPS Library from Arduiniana
#include <maidenhead.h> // The Maindenhead Grid Square Library

#define audio_enable 6    // Audio Enable switch on Pin 6

#define gps_rxPin 4    // Serial input (connects to GPS TxD)
#define gps_txPin -1    // Serial output (connects to GPS RxD) Set to -1 (disabled) because we don't need to send anything to GPS

#define baud_rate 115200  // Defines the Sweial Monitor baud rate
#define gps_baud_rate 9600

#define tft_cyan ILI9341_CYAN // Simplifies the color definitions for the TFT
#define tft_black ILI9341_BLACK
#define tft_red ILI9341_RED
#define tft_green ILI9341_GREEN
#define tft_yellow ILI9341_YELLOW

SoftwareSerial gps_serial(gps_rxPin, gps_txPin);  // set up a new software serial port for the GPS

TinyGPSPlus gps;  // set up a new GPS object

float LAT, LON;  // floating integer for latitude and longitude values
double gps_altitude, prev_gps_altitude = -9999;   // Variable to hold the altitude value
int gps_tick = 0, gps_timeout = 240;  // GPS startup timer variables
unsigned long gps_time, gps_date; // GPS date and time variables
String s_date, s_time, s_month, s_year, s_day, s_hour, s_minute, s_second, grid_text; // String variables for date, time and grid square
boolean gps_ok = false;  // gps data ok flag
bool audio_on = false; // Speech enabled flag
bool prev_audio;  // Previous state of speech enabled flag
int num_sats = 0; // Number of satellites being received
int prev_sats = -1;  // Previous number received for status update
String prev_date = "", prev_time = " ", prev_grid = ""; // variables to hold the previous info
float prev_LAT = 999, prev_LON = 999;  // floating integer for previous latitude and previous longitude values
String msg; // The message to send to the Speech Synthesizer

// Note: Value set to 999 so it will update on first pass

#define font_size 8  // The text display font size
#define status_font_size 2  // The font size for the status line

#define TFT_CS 10 // Defines the CS pin for the TFT
#define TFT_DC 7  // Defines the DC pin for the TFT

int background = tft_black; // Set the TFT background and text color
int text_color = tft_cyan;

// Use hardware SPI for TFT
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);  // Create the TFT instance

void setup()
{

  Serial.begin(baud_rate);  // Must have or Text to Speech module library hangs
  Serial.println("Starting Serial Port");

  ss.begin();  //Init speech synthesis sensor

  ss.setVolume(3);  //Set voice volume to 3
  ss.setSpeed(5);  //Set playback speed to 5
  ss.setSoundType(ss.eFemale1);  //Set speaker to female
  ss.setTone(4);  //Set tone to 4
  ss.setEnglishPron(ss.eWord);  //For English, speak word

  pinMode(audio_enable, INPUT_PULLUP);  // Setup the Audio enable pin as an input

  start_tft();  // Start the TFT display

  gps_serial.begin(gps_baud_rate);  // set the data rate for the GPS SoftwareSerial port

  // Check for valid GPS data
  clear_display();  // Clear the TFT display
  tft.setCursor(70, 50);  // Display the aquiring satellites message
  tft.print("Acquiring");
  tft.setCursor(60, 90);
  tft.print("Satellites");

  while ((gps.satellites.value() == 0) && (gps_tick < gps_timeout)) // Loop until we start getting satellites
  {
    num_sats = gps.satellites.value();

#ifdef debug
    Serial.print("Sats: "); Serial.print(num_sats); Serial.print("   Location Valid: "); Serial.println(gps.location.isValid());
#endif

    tft.setCursor(95, 150);
    tft.print("  " + String(gps_timeout - gps_tick) + "  "); // Display the timeout timer
    smartDelay(1000); // Read and update GPS data
    gps_tick = gps_tick + 1;  // Wait a second and decrement the timeout timer
  }
  if (gps_tick < gps_timeout)  // Check to see if valid message
  {
    gps_ok = true;  // We got valid data before timeout, flag the GPS data as receiving satellites
    num_sats = gps.satellites.value();

#ifdef debug
    Serial.println("Receiving Satellites");
    Serial.print("Sats: "); Serial.print(num_sats); ; Serial.print("   Location Valid: "); Serial.println(gps.location.isValid());
#endif
    clear_display();  // Clear the TFT display
    tft.setCursor(60, 50);  // Display the waiting for enough satellites to determine position
    tft.print("Waiting For");
    tft.setCursor(80, 90);
    tft.print("Satellite");
    tft.setCursor(130, 130);
    tft.print("Lock");
    smartDelay(30000);
    clear_display();
    update_status();

    // Set the previous audio enable flag to the opposite of what the enable switch reads
    prev_audio = (digitalRead(audio_enable)); // Read the state of the audio enable switch
  } else
  {
    // GPS Failed to lock
    clear_display();
    tft.setCursor(130, 50); // If we get here, the GPS failed to get valid satellite data after the timeout period
    tft.print("GPS");
    tft.setCursor(100, 90);
    tft.print("Failed");
    tft.setCursor(95, 130);
    tft.print("To Lock");

    while (true);  // Loop here until reset
    {
    }
  }

  // Set the speech enable to the opposite of what it is now so it will display correctly
  prev_audio = digitalRead(audio_enable); // Read the state of the audio enable switch

}  // End Setup Loop

void loop()    // Start the Main Loop
{
  audio_on = !(digitalRead(audio_enable)); // Read the state of the audio enable switch

#ifdef debug
  Serial.print("Audio Enable: ");
  Serial.print(audio_on);
  Serial.print("  Prev: ");
  Serial.println(prev_audio);
#endif

  smartDelay(0);  // smartDelay is used by GPS library to update satellite info while delating x milliseconds
  num_sats = gps.satellites.value();  // Get the number of satellites
  LAT = gps.location.lat();  // Get the latitude
  LON = gps.location.lng();  // Get the longitude

#ifdef debug
  Serial.print("Sats: "); Serial.print(num_sats);
  Serial.print("   LAT: "); Serial.print(LAT, 9);
  Serial.print("   LON: "); Serial.print(LON, 9);
  Serial.print("   Location Valid: "); Serial.println(gps.location.isValid());
#endif
  smartDelay (3000);

  gps_date = gps.date.value(); // Read the GPS Date info
  s_date = gps_date;  // Put the date in a separate string

  gps_time = gps.time.value(); // Read the GPS Time info
  s_time = gps_time;  // Put the time in a separate string

#ifdef debug
  Serial.print("Date: "); Serial.print(gps.date.value()); Serial.print("   Time: "); Serial.println(gps.time.value());
#endif

  if (s_date.length() < 6)  // If the date string length is less than 6, pad it until it is
  {
    s_date = "0" + s_date;
  }

  while (s_time.length() < 8)  // If the time string length is less than 8, pad it until it is
  {
    s_time = "0" + s_time;
  }

  s_year = s_date.substring(4, 6); // Break out the date string into Day/Month/Year
  s_month = s_date.substring(2, 4);
  s_day = s_date.substring(0, 2);
  s_hour = s_time.substring(0, 2); // Break out the time string into Hour/Minute/Second
  s_minute = s_time.substring(2, 4);
  s_second = s_time.substring(4, 6);

  gps_altitude = gps.altitude.feet();  // Get the altitude

#ifdef debug
  Serial.print("Date: "); Serial.print(gps_date); Serial.print("  Date String: "); Serial.print(s_date); Serial.print("   Length: "); Serial.print(s_date.length());
  Serial.print("   Time: "); Serial.print(gps_time); Serial.print("   Time String: "); Serial.print(s_time); Serial.print(" Altitude: "); Serial.println(gps.altitude.feet());
#endif

  grid_text = get_mh(LAT, LON, 6);  // Get the Maidenhead Grid Square

#ifdef debug
  Serial.print("Grid Square: ");
  Serial.print(grid_text);
  Serial.print("  Length: ");
  Serial.println(grid_text.length());
#endif

  update_status();  // Update the TFT display

  // The Text to Speech part of the sketch

  if (audio_on)  // Only speak if Audio is enabled
  {
    ss.speak("[g2][h2]U T C Time is");  // Say the Time
    msg = ("\"[n1]" + s_hour + s_minute + "\"");  // Prepare the message
    ss.speak(msg);  // Speak the Time

    ss.speak("Grid Square");  // Say the Grid Square
    msg = ("\"" + grid_text + "\"");  // Prepare the message
    ss.speak(msg);  // Speak the grid square
  }

  smartDelay(10000);  // Delay 10 seconds and update the GPS info

}  // End Main Loop

void start_tft()  // Starts the TFT display and shows the startup screen
{
  tft.begin();  // Start the TFT display

#ifdef debug
  Serial.println("TFT Start");  // Verifies that the TFT is operational
  Serial.print("Display Pwr Mode: 0x");
  Serial.println(tft.readcommand8(ILI9341_RDMODE), HEX);
#endif

  clear_display();  // Clear the TFT display
  tft.setTextColor(text_color, background);  // Set the display text to black. Green background color required to "clear" previous text
  tft.setRotation(3); // horizontal display // change rotation as needed/desired
  tft.setTextSize(3);  // large font
  tft.setCursor(110, 50);  //Set the Cursor and display the startup screen
  tft.print("KW5GP");
  tft.setCursor(50, 90);
  tft.print("Talking Grid");
  tft.setCursor(30, 130);
  tft.print("Square Display");

  delay(5000);  //Wait 5 seconds then clear the startup message
  clear_display();  // Clear the TFT display
}

void clear_display()  // Function to clear the TFT display
{
  tft.fillScreen(background); // Clear the displayusing current background color
}

// This custom version of delay() ensures that the GPS object is being "fed".

static void smartDelay(unsigned long ms)
{
  unsigned long start = millis();
  do
  {
    while (gps_serial.available())
      gps.encode(gps_serial.read());
  } while (millis() - start < ms);
}

void update_status()
{
  if (prev_grid != grid_text)
  {
    // Display the Grid Square
    tft.setTextSize(font_size);
    tft.setCursor(20, 30);
    tft.print(grid_text);
  }
  tft.setTextSize(status_font_size);
  // Draw lines for config/status area
  tft.drawFastHLine(0, 115, 320, tft_cyan);
  //tft.drawFastVLine(140, 185, 45, tft_cyan);

  if (num_sats != prev_sats)
  {
    tft.setCursor(5, 220);
    tft.print("Sats: ");

    tft.print(num_sats);
    tft.print("  ");
    prev_sats = num_sats;
  }

#ifdef debug
  Serial.print("Checking for switch change Audio: ");
  Serial.print(audio_on);
  Serial.print("  Prev: ");
  Serial.println(prev_audio);
#endif

  if (prev_audio != audio_on)
  {

#ifdef debug
    Serial.println("Switch has changed");
#endif

    // Audio enable switch has changed
    tft.setCursor(180, 220);
    tft.print("Speech: ");
    if (audio_on)
    {
      // Audio is now enabled
      tft.print("On ");

    } else
    {
      // Audio is now disabled
      tft.print("Off");
    }
    prev_audio = audio_on;
  }

  // Display the date
  if (gps.date.isValid() && s_date.length() == 6 && s_date != prev_date)
  {
    tft.setCursor (5, 120);
    tft.print("Date: ");
    tft.print(s_month + " / " + s_day + " / " + s_year); // Display the Date
    prev_date = s_date;
  }

  // Display the time
  if (gps.time.isValid() && s_time.length() == 8 && s_time != prev_time)
  {
    tft.setCursor (5, 140);
    tft.print("Time: ");
    tft.print(s_hour + ": " + s_minute + ": " + s_second); // Display the Time
    tft.print(" UTC");
    prev_time = s_time;
  }

  if (gps.location.isValid() && gps_ok)
  {

    if (LAT != prev_LAT)  // Update the latitude if it has changed
    {
      tft.setCursor(5, 160);
      tft.print("Lat: ");
      if (LAT >= 0)
      {
        tft.print(" ");
      }
      tft.print(LAT, 9);
      prev_LAT = LAT;
    }
    
    if (LON != prev_LON)  // Update the longitude if it has changed
    {
      tft.setCursor(5, 180);
      tft.print("Lon: ");
      if (LON >= 0)
      {
        tft.print(" ");
      }
      tft.print(LON, 9);
      prev_LON = LON;
    }
    
    if (gps_altitude != prev_gps_altitude)  // Update the altitude if it has changed
    {
      // Altitude has changed
      tft.setCursor(5, 200);
      tft.print("Altitude: ");
      tft.print(gps_altitude, 1);
      tft.print(" Feet     ");
      prev_gps_altitude = gps_altitude;
    }
  }
  tft.setTextSize(font_size);
}
