

/*
  Talking SWR Meter by Glen Popiel - KW5GP

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 3 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

/*
   SWR RF head
  Forward connected to Analog 0
  Reflected connected to Analog 1

  ILI9341 TFT module is connected to the following pins.
      CLK  - Pin 12
      MOSI  - Pin 11
      CS   - Pin 10
      DC   - Pin 7
*/

//#define debug   // Enables diagnostic information

#include "DFRobot_SpeechSynthesis_V2.h"
DFRobot_SpeechSynthesis_I2C ss;

#include <SPI.h>  // SPI library required for SPI devices
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ILI9341.h>

#define TFT_CS     10  // Assign the TFT CS to pin 10
#define TFT_DC     7  // Assign the TFT DC to pin 8

#define audio_enable 6  // Audio Enable switch input

#define baud_rate 115200  // Define the Serial Monitor baud rate

#define tft_cyan ILI9341_CYAN
#define tft_black ILI9341_BLACK
#define tft_red ILI9341_RED
#define tft_green ILI9341_GREEN
#define tft_yellow ILI9341_YELLOW

int background = tft_green;

// Use hardware SPI (on Uno/Nano, #13, #12, #11) and the above for CS/DC
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);  // Create the TFT instance

#define adc_count 0.0048828125  // Volts per ADC count

float V_Fwd, V_Ref = 0, V_SWR ; // The variables to contain the SWR values
bool audio_on = false; // Speech enabled flag
bool prev_audio;  // Previous state of speech enabled flag
String msg; // The message to send to the Speech Synthesizer
String s_SWR; // Holds the string representation of V_SWR
int x_SWR;
int (update_time) = 10000;

void setup()
{
#ifdef debug
  Serial.begin(baud_rate);  // Required for DFRobot TTS module
  Serial.println("Starting Serial Port for debugging");
#endif

  ss.begin();  //Init speech synthesis sensor

  ss.setVolume(3);  //Set voice volume to 3
  ss.setSpeed(7);  //Set playback speed to 7
  ss.setSoundType(ss.eFemale1);  //Set speaker to female
  ss.setTone(4);  //Set tone to 4
  ss.setEnglishPron(ss.eWord);  //For English, speak word

  pinMode(audio_enable, INPUT_PULLUP);  // Setup the Audio enable pin as an input

  // Set the previous audio enable flag to the opposite of what the enable switch reads
  prev_audio = (digitalRead(audio_enable)); // Read the state of the audio enable switch

  start_tft();  // Start the TFT display

}  // End Setup Loop

void loop()    // Start the Main Loop
{
  // Read the audio enable switch
  audio_on = !(digitalRead(audio_enable)); // Read the state of the audio enable switch

#ifdef debug
  Serial.print("Audio Enable: ");
  Serial.print(audio_on);
  Serial.print("  Prev: ");
  Serial.println(prev_audio);
#endif

  // Read the A/D converters
  V_Fwd = analogRead(0) * adc_count;
  V_Ref = analogRead(1) * adc_count;

  // Calculate VSWR
  if (V_Fwd > V_Ref)
  {
    V_SWR = (V_Fwd + V_Ref) / (V_Fwd - V_Ref);
  }
  else if (V_Fwd == 0 && V_Ref == 0)
  {
    V_SWR = 1;
  }
  else
  {
    V_SWR = 10;
  }
  if (V_SWR > 10)
  {
    V_SWR = 10;
  }

  background = tft_green;
  if (V_SWR > 2)
  {
    // Turn the background yellow
    background = tft_yellow;
  }
  if (V_SWR > 3)
  {
    // Turn the background yellow
    background = tft_red;
  }

#ifdef debug1
  Serial.print("V_SWR = ");
  Serial.print(V_SWR);
#endif

  x_SWR = (V_SWR * 10); // Trim the value to 1 decimal place
  V_SWR = float(x_SWR) / 10;

  tft.setTextColor(tft_black, background);
  clear_display();
  tft.setCursor(120, 5); //
  tft.setTextSize(4);  // large font
  tft.print("SWR");
  tft.setTextSize(6);  // large font
  tft.setCursor(70, 60);
  tft.print(V_SWR, 1);
  tft.print(":1");
  tft.setTextSize(3);
  tft.setCursor(20, 140);
  tft.print("Fwd V: ");
  tft.print(V_Fwd, 6);
  tft.setCursor(20, 170); //Set the cursor to the top
  tft.print("Ref V: ");
  tft.print(V_Ref, 6);

  // Display the status of the Speech Enable Switch
  tft.setTextSize(3);  // large font
  tft.setCursor(70, 210);
  tft.print("Speech: ");
  if (audio_on)
  {
    // Audio is now enabled
    tft.print("On ");
  } else
  {
    // Audio is now disabled
    tft.print("Off");
  }
  prev_audio = audio_on;

  // The Text to Speech part of the sketch

  s_SWR = String(V_SWR);  // Convert V_SWR to String
  s_SWR = s_SWR.substring(0, s_SWR.length() - 1); // Trim to one decimal place

#ifdef debug1
  Serial.print("   x_SWR = ");
  Serial.print(x_SWR);
  Serial.print("   New V_SWR = ");
  Serial.print(V_SWR);
  Serial.print("   s_SWR = ");
  Serial.println(s_SWR);
#endif


  if (audio_on)  // Only speak if Audio is enabled
  {
    if (V_SWR > 3)
    {
      ss.speak("[v3][s7]Warnning"); // SWR is over 3.0 - speak warning (have to tweak the phonetics)
    }

    ss.speak("[v3][g2][h2][s7]S W R is ");  // Send the desired string to convert to speech
    msg = ("\"[v3][n2][s7]" + s_SWR + "to:1" + "\""); // Prepare the message
    ss.speak(msg);  // Speak the SWR

  }

  delay(update_time);
}

void start_tft()  // Starts the TFT display and shows the startup screen
{
  tft.begin();  // Start the TFT display

#ifdef debug

  Serial.println("TFT Start");  // Verifies that the TFT is operational
  Serial.print("Display Pwr Mode: 0x");
  Serial.println(tft.readcommand8(ILI9341_RDMODE), HEX);
#endif

  clear_display();  // Clear the TFT display
  tft.setTextColor(tft_black, tft_green);  // Set the display text to black. Green background color required to "clear" previous text
  tft.setRotation(3); // horizontal display // change rotation as needed/desired
  tft.setTextSize(4);  // large font
  tft.setCursor(100, 40);  //Set the Cursor and display the startup screen
  tft.print("KW5GP");
  tft.setCursor(70, 90);
  tft.print("Talking");
  tft.setCursor(50, 140);
  tft.print("SWR Meter");

  delay(5000);  //Wait 5 seconds then clear the startup message
  clear_display();  // Clear the TFT display
}


void clear_display()  // Function to clear the TFT display
{
  tft.fillScreen(background); // Clear the displayusing current background color
}
