/* Yeasu GS-232/G5400B Rotor Controller
  Replaces Yeasu GS-232A
  Written by Glen Popiel, KW5GP

  ILI9341 TFT module is connected to the following pins.
    CLK  - Pin 12
    MOSI  - Pin 11
    CS   - Pin 10
    DC   - Pin 9
    RST - connected to Vcc pin
    LED - connected to Vcc pin
*/

//#define debug // Enables debug mode for A/D functions

//#define debug1 // Enables debug mode for TFT

#define HRD // Defined if we're using HRD - we need to disable character echo

#include <Adafruit_ADS1X15.h> // Include the adafruit ADS1X15 A/D library
#include <EEPROM.h> // Include the EEPROM library so we can save calibration data
#include <SPI.h>
#include <Adafruit_GFX.h> // The Adafruit Graphics Library
#include <Adafruit_ILI9341.h> // The ILI9341 TFT Display Library

Adafruit_ADS1115 adc;  /* Use this for the 16-bit version */


#define rotate_down 3
#define rotate_up 4
#define rotate_left 5
#define rotate_right 6

#define font_size 6  // The TFT text display font size
#define status_font_size 2  // The TFT font size for the status line

#define TFT_CS 10 // Defines the CS pin for the TFT
#define TFT_DC 9  // Defines the DC pin for the TFT

#define BAUD_RATE 19200 // Max Baud Rate is 19200 if using Ham Radio Deluxe, 57600 if using SatPC32

#define tft_cyan ILI9341_CYAN // Simplifies the color definitions for the TFT
#define tft_black ILI9341_BLACK
#define tft_red ILI9341_RED
#define tft_green ILI9341_GREEN
#define tft_yellow ILI9341_YELLOW

#define EEPROM_ID_BYTE 1   // EEPROM ID to validate EEPROM data location
#define EEPROM_ID  55  // EEPROM ID Value
#define EEPROM_AZ_CAL_0 2    // Azimuth Zero Calibration EEPROM location     
#define EEPROM_AZ_CAL_MAX 4  // Azimuth Max Calibration Data EEPROM location  
#define EEPROM_EL_CAL_0 6    // Elevation Zero Calibration Data EEPROM location  
#define EEPROM_EL_CAL_MAX 8  // Elevation Max Calibration Data EEPROM location  

#define AZ_CAL_0_DEFAULT 0
#define AZ_CAL_MAX_DEFAULT 27000
#define EL_CAL_0_DEFAULT 0
#define EL_CAL_MAX_DEFAULT 27000

#define AZ_Tolerance 3  // The G5400 controller needs some tolerance because
#define EL_Tolerance 3  // of the power supply drop during rotation

//variables
byte inByte = 0;  // incoming serial byte
byte serial_buffer[50];  // incoming serial byte buffer
int serial_buffer_index = 0;
int set_AZ;  // Azimuth set value
int set_EL;  // Elevation set value
int current_AZ;  // Current Azimuth raw value
int current_EL;  // Current Elevation raw value
String Serial_Send_Data; // Data to send to Serial Port
int AZ_0;  // Azimuth Zero Value from EEPROM
int AZ_MAX; // Azimuth Max Value from EEPROM
int EL_0;  // Elevation 0 Value from EEPROM
int EL_MAX;  //Elevation Max Value from EEPROM
int AZ_Degrees; // mapped AZ ADC value to Degrees
int EL_Degrees; // mapped EL ADC value to Degrees
String Requested_AZ; // RS232 Requested Azimuth - M and short W command
String Requested_EL; //RS232 Requested Azimuth and Elevation - Full W command
int AZ_To; // Requested AZ Move
int EL_To; // Requested EL Move
int AZ_Distance; // Distance to move AZ
int EL_Distance; // Distance to move EL

int prev_AZ_Degrees = -1;  // The previous reading so we can update the TFT
int prev_EL_Degrees = -1; // Set to -1 so it will trigger an update

int background = tft_black; // Set the TFT background and text color
int text_color = tft_cyan;

// Use hardware SPI for TFT
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);  // Create the TFT instance

void setup()
{
  pinMode(rotate_up, OUTPUT);
  pinMode(rotate_down, OUTPUT);
  pinMode(rotate_left, OUTPUT);
  pinMode(rotate_right, OUTPUT);

  digitalWrite(rotate_up, LOW);
  digitalWrite(rotate_down, LOW);
  digitalWrite(rotate_left, LOW);
  digitalWrite(rotate_right, LOW);

  start_tft();  // Start the TFT display

  Serial.begin(BAUD_RATE); // initialize serial communication on the USB port

  adc.begin();  // Start the ADS1115 A/D

  adc.setGain(GAIN_TWOTHIRDS);  // 2/3x gain +/- 6.144V  1 bit = 3mV      0.1875mV (default)

  set_AZ = -1;
  set_EL = -1;

  read_eeprom_cal_data();

  update_tft();

  // End Setup
}

void loop()
{
  check_serial();
  check_move();
  update_tft();

  // End Main Loop
}

// Functions

void read_eeprom_cal_data()
{
  if (EEPROM.read(EEPROM_ID_BYTE) == EEPROM_ID)
  {

#ifdef debug
    Serial.println("Read EEPROM Calibration Data Valid ID");
    Serial.println((EEPROM.read(EEPROM_AZ_CAL_0) * 256) + EEPROM.read(EEPROM_AZ_CAL_0 + 1), DEC);
    Serial.println((EEPROM.read(EEPROM_AZ_CAL_MAX) * 256) + EEPROM.read(EEPROM_AZ_CAL_MAX + 1), DEC);
    Serial.println((EEPROM.read(EEPROM_EL_CAL_0) * 256) + EEPROM.read(EEPROM_EL_CAL_0 + 1 ), DEC);
    Serial.println((EEPROM.read(EEPROM_EL_CAL_MAX) * 256) + EEPROM.read(EEPROM_EL_CAL_MAX + 1), DEC);
#endif

    AZ_0 = (EEPROM.read(EEPROM_AZ_CAL_0) * 256) + EEPROM.read(EEPROM_AZ_CAL_0 + 1);
    AZ_MAX = (EEPROM.read(EEPROM_AZ_CAL_MAX) * 256) + EEPROM.read(EEPROM_AZ_CAL_MAX + 1);
    EL_0 = (EEPROM.read(EEPROM_EL_CAL_0) * 256) + EEPROM.read(EEPROM_EL_CAL_0 + 1);
    EL_MAX = (EEPROM.read(EEPROM_EL_CAL_MAX) * 256) + EEPROM.read(EEPROM_EL_CAL_MAX + 1);
  } else
  { // initialize eeprom to default values

#ifdef debug
    Serial.println("Read EEPROM Calibration Data Invalid ID - setting to defaults");
#endif

    AZ_0 = AZ_CAL_0_DEFAULT;
    AZ_MAX = AZ_CAL_MAX_DEFAULT;
    EL_0 = EL_CAL_0_DEFAULT;
    EL_MAX = EL_CAL_MAX_DEFAULT;
    write_eeprom_cal_data();
  }
}

void write_eeprom_cal_data()
{
#ifdef debug
  Serial.println("Writing EEPROM Calibration Data");
#endif

  EEPROM.write(EEPROM_ID_BYTE, EEPROM_ID);
  EEPROM.write(EEPROM_AZ_CAL_0, highByte(AZ_0));
  EEPROM.write(EEPROM_AZ_CAL_0 + 1, lowByte(AZ_0));
  EEPROM.write(EEPROM_AZ_CAL_MAX, highByte(AZ_MAX));
  EEPROM.write(EEPROM_AZ_CAL_MAX + 1, lowByte(AZ_MAX));
  EEPROM.write(EEPROM_EL_CAL_0, highByte(EL_0));
  EEPROM.write(EEPROM_EL_CAL_0 + 1, lowByte(EL_0));
  EEPROM.write(EEPROM_EL_CAL_MAX, highByte(EL_MAX));
  EEPROM.write(EEPROM_EL_CAL_MAX + 1, lowByte(EL_MAX));
}

void check_serial()
{
  if (Serial.available() > 0)
  {
    inByte = Serial.read();

#ifndef HRD // Echo the command if we're not using HRD
    Serial.print(char(inByte));
#endif

    if (inByte == 10)
    {
      return; // ignore Line Feeds
    }
    if (inByte != 13) { // Add to buffer if not CR
      serial_buffer[serial_buffer_index] = inByte;

#ifdef debug
      Serial.print(" Received = ");
      Serial.println(serial_buffer[serial_buffer_index]);
#endif

      serial_buffer_index++;

    } else
    {

      // It's a CR, execute command

      if ((serial_buffer[0] > 96) && (serial_buffer[0] < 123))
      { // it's lowercase, convert to uppercase
        serial_buffer[0] = serial_buffer[0] - 32;
      }


      switch (serial_buffer[0]) {  // Decode first character of command

        case 65:

#ifdef debug
          Serial.println("A Command Received");
#endif

          az_rotate_stop();
          break;

        case 66:

#ifdef debug
          Serial.println("B Command Received");
#endif
          send_current_el();
          break;

        case 67:      // C - return current azimuth

#ifdef debug
          Serial.println("C Command Received");
          Serial.println(serial_buffer_index);
#endif

          if ((serial_buffer_index == 2) & (serial_buffer[1] == 50)) {

#ifdef debug
            Serial.println("C2 Command Received");
#endif

            send_current_azel();
          } else {
            send_current_az();
          }
          break;

        case 68:

#ifdef debug
          Serial.println("D Command Received");
#endif

          rotate_el_down();
          break;

        case 69:

#ifdef debug
          Serial.println("E Command Received");
#endif
          el_rotate_stop();
          break;

        case 70:

#ifdef debug
          Serial.println("F Command Received");
          Serial.println(serial_buffer_index);
#endif

          if ((serial_buffer_index == 2) & (serial_buffer[1] == 50)) {

#ifdef debug
            Serial.println("F2 Command Received");
#endif

            set_max_el_cal();
          } else {
            set_max_az_cal();
          }
          break;

        case 76:

#ifdef debug
          Serial.println("L Command Received");
#endif
          rotate_az_ccw();
          break;

        case 77:

#ifdef debug
          Serial.println("M Command Received");
#endif

          rotate_to();
          break;

        case 79:

#ifdef debug
          Serial.println("O Command Received");
          Serial.println(serial_buffer_index);
#endif
          if ((serial_buffer_index == 2) & (serial_buffer[1] == 50))
          {
#ifdef debug
            Serial.println("O2 Command Received");
#endif

            set_0_el_cal();
          } else
          {
            set_0_az_cal();
          }
          break;

        case 82:

#ifdef debug
          Serial.println("R Command Received");
#endif

          rotate_az_cw();
          break;

        case 83:

#ifdef debug
          Serial.println("S Command Received");
#endif

          az_rotate_stop();
          el_rotate_stop();
          break;

        case 85:

#ifdef debug
          Serial.println("U Command Received");
#endif

          rotate_el_up();
          break;

        case 87:

#ifdef debug
          Serial.println("W Command Received");
#endif

          rotate_az_el_to();
          break;

      }

      serial_buffer_index = 0;
      serial_buffer[0] = 0;

    }
  }
}

void send_current_az()
{
  read_adc();
  // Map it to degrees

#ifdef debug
  Serial.println(current_AZ);
#endif

  AZ_Degrees = map(current_AZ, AZ_0, AZ_MAX, 0, 360);
  if (AZ_Degrees > 180) {
    AZ_Degrees = AZ_Degrees - 180;
  } else
  {
    AZ_Degrees = AZ_Degrees + 180;
  }

#ifdef debug
  Serial.println(AZ_Degrees);
#endif

  // Send it back via serial
  Serial_Send_Data = "";
  if (AZ_Degrees < 100)
  { // pad with 0's if needed
    Serial_Send_Data = "0";
  }
  if (AZ_Degrees < 10)
  {
    Serial_Send_Data = "00";
  }
  Serial_Send_Data = "+0" + Serial_Send_Data + String(AZ_Degrees);
  Serial.println(Serial_Send_Data);  // Return value via RS-232 port

}

void send_current_azel()
{
  read_adc();
  // Map AZ to degrees

#ifdef debug
  Serial.println(current_AZ);
#endif

  AZ_Degrees = map(current_AZ, AZ_0, AZ_MAX, 0, 360);
  if (AZ_Degrees > 180)
  {
    AZ_Degrees = AZ_Degrees - 180;
  } else {
    AZ_Degrees = AZ_Degrees + 180;
  }

  // Map EL to degrees
  EL_Degrees = map(current_EL, EL_0, EL_MAX, 0, 180);

#ifdef debug
  Serial.println(current_EL);
  Serial.println(EL_Degrees);
  Serial.println(AZ_Degrees);
#endif

  // Send it back via serial
  Serial_Send_Data = "";

  if (AZ_Degrees < 100)
  { // pad with 0's if needed
    Serial_Send_Data = "0";
  }
  if (AZ_Degrees < 10)
  {
    Serial_Send_Data = "00";
  }
  Serial_Send_Data = "+0" + Serial_Send_Data + String(AZ_Degrees) + "+0";

#ifdef HRD
  if (EL_Degrees > 90) // Limit elevation response to 90 degrees for HRD
  {
    EL_Degrees = 90;
  }
#endif

  if (EL_Degrees < 100)
  { // pad with 0's if needed
    Serial_Send_Data = Serial_Send_Data + "0";
  }

  if (EL_Degrees < 10)
  {
    Serial_Send_Data = Serial_Send_Data + "0";
  }

  Serial_Send_Data = Serial_Send_Data + String(EL_Degrees);
  Serial.println(Serial_Send_Data);  // Return value via RS-232 port
}

void set_max_az_cal() {

#ifdef debug
  Serial.println("Calibrate Max AZ Function");
#endif

  read_adc();
  // save current az and el values to EEPROM - Zero Calibration

#ifdef debug
  Serial.println(current_AZ);
#endif

  AZ_MAX = current_AZ;
  write_eeprom_cal_data();
  Serial.println("Max Azimuth Calibration Complete");
}

void set_max_el_cal() {

#ifdef debug
  Serial.println("Calibrate EL Max Function");
#endif

  read_adc();
  // save current az and el values to EEPROM - Zero Calibration

#ifdef debug
  Serial.println(current_EL);
#endif

  EL_MAX = current_EL;
  write_eeprom_cal_data();
  Serial.println("Max Elevation Calibration Complete");
}

void rotate_az_ccw()
{
  digitalWrite(rotate_left, HIGH);
  digitalWrite(rotate_right, LOW);
}

void rotate_az_cw()
{
  digitalWrite(rotate_right, HIGH);
  digitalWrite(rotate_left, LOW);
}

void rotate_el_up()
{
  digitalWrite(rotate_up, HIGH);
  digitalWrite(rotate_down, LOW);
}

void rotate_el_down()
{
  digitalWrite(rotate_down, HIGH);
  digitalWrite(rotate_up, LOW);
}

void az_rotate_stop()
{
  digitalWrite(rotate_right, LOW);
  digitalWrite(rotate_left, LOW);
}

void el_rotate_stop()
{
  digitalWrite(rotate_up, LOW);
  digitalWrite(rotate_down, LOW);
}

void rotate_to()
{

  // Decode Command - Format Mxxx - xxx = Degrees to Move to

#ifdef debug
  Serial.println("M Command -  Rotate Azimuth To Function");
  Serial.println(serial_buffer_index);
#endif

  if (serial_buffer_index == 4)
  {

#ifdef debug
    Serial.println("Value in [1] to [3]?");
#endif

    Requested_AZ = (String(char(serial_buffer[1])) + String(char(serial_buffer[2])) + String(char(serial_buffer[3]))) ;
    AZ_To = (Requested_AZ.toInt()); // AZ Degrees to Move to as integer

    if (AZ_To < 0)
    {
      AZ_To = 0;
    }

    if (AZ_To > 360)
    {
      AZ_To = 360;
    }

    if (AZ_To > 180)
    {
      AZ_To = AZ_To - 180;
    } else
    {
      AZ_To = AZ_To + 180;
    }

#ifdef debug
    Serial.println(Requested_AZ);
    Serial.println(AZ_To);
#endif

    // set the move flag and start
    read_adc();
    // Map it to degrees
    AZ_Degrees = map(current_AZ, AZ_0, AZ_MAX, 0, 360);

#ifdef debug
    Serial.println(current_AZ);
    Serial.println(AZ_Degrees);
#endif

    AZ_Distance = AZ_To - AZ_Degrees;
    set_AZ = AZ_To;
    if (abs(AZ_Distance) <= AZ_Tolerance) { // No move needed
      az_rotate_stop();
      set_AZ = -1;
    } else
    { // Move Azimuth - figure out which way
      if (AZ_Distance > 0)
      { //We need to move CCW
        rotate_az_ccw();
      } else {
        rotate_az_cw();
      }
    }
  }
}

void send_current_el()
{
  read_adc();
  // Map it to degrees
  EL_Degrees = map(current_EL, EL_0, EL_MAX, 0, 180);

#ifdef debug
  Serial.println(current_EL);
  Serial.println(EL_Degrees);
  Serial.println(AZ_Degrees);
#endif

  // Send it back via serial
  Serial_Send_Data = "";

  if (EL_Degrees < 100)
  { // pad with 0's if needed
    Serial_Send_Data = "0";
  }

  if (EL_Degrees < 10)
  {
    Serial_Send_Data = "00";
  }

  Serial_Send_Data = "+0" + Serial_Send_Data + String(EL_Degrees);
  Serial.println(Serial_Send_Data);  // Return value via RS-232 port
}

void rotate_az_el_to()
{
  // Decode Command - Format Wxxx yyy where xxx = Azimuth to Move to yyy = Elevation to Move to

#ifdef debug
  Serial.println("W Command -  Rotate Azimuth and Elevation To Function");
  Serial.println(serial_buffer_index);
#endif

  if (serial_buffer_index == 8)
  {

#ifdef debug
    Serial.println("Value in [1] to [3]?");
#endif

    Requested_AZ = (String(char(serial_buffer[1])) + String(char(serial_buffer[2])) + String(char(serial_buffer[3]))) ;
    AZ_To = (Requested_AZ.toInt()); // AZ Degrees to Move to as integer
    if (AZ_To < 0)
    {
      AZ_To = 0;
    }

    if (AZ_To > 360)
    {
      AZ_To = 360;
    }

    if (AZ_To > 180)
    {
      AZ_To = AZ_To - 180;
    } else
    {
      AZ_To = AZ_To + 180;
    }

#ifdef debug
    Serial.println(Requested_AZ);
    Serial.println(AZ_To);
    Serial.println("Value in [5] to [7]?");
#endif

    Requested_EL = (String(char(serial_buffer[5])) + String(char(serial_buffer[6])) + String(char(serial_buffer[7]))) ;
    EL_To = (Requested_EL.toInt()); // EL Degrees to Move to as integer
    if (EL_To < 0)
    {
      EL_To = 0;
    }

    if (EL_To > 180)
    {
      EL_To = 180;
    }

#ifdef debug
    Serial.println(Requested_EL);
    Serial.println(EL_To);
#endif

    // set the move flag and start
    read_adc();
    // Map it to degrees
    AZ_Degrees = map(current_AZ, AZ_0, AZ_MAX, 0, 360);

#ifdef debug
    Serial.println(current_AZ);
    Serial.println(AZ_Degrees);
#endif

    AZ_Distance = AZ_To - AZ_Degrees;
    set_AZ = AZ_To;
    EL_Degrees = map(current_EL, EL_0, EL_MAX, 0, 180);

#ifdef debug
    Serial.println(EL_Degrees);
#endif

    EL_Distance = EL_To - EL_Degrees;
    set_EL = EL_To;
    // Set Azimuth
    if (abs(AZ_Distance) <= AZ_Tolerance)
    { // No move needed
      az_rotate_stop();
      set_AZ = -1;
    } else
    { // Move Azimuth - figure out which way
      if (AZ_Distance > 0) {  //We need to move CW
        rotate_az_ccw();
      } else {
        rotate_az_cw();
      }
    }
    // Set Elevation
    if (abs(EL_Distance) <= EL_Tolerance)
    { // No move needed
      el_rotate_stop();
      set_EL = -1;
    } else
    { // Move Azimuth - figure out which way
      if (EL_Distance > 0)
      { //We need to move CW
        rotate_el_up();
      } else
      {
        rotate_el_down();
      }
    }
  }
}

void set_0_az_cal()
{
  read_adc();
  // save current az and el values to EEPROM - Zero Calibration

#ifdef debug
  Serial.println("Calibrate Zero Function");
  Serial.println(current_EL);
  Serial.println(current_AZ);
#endif

  AZ_0 = current_AZ;
  write_eeprom_cal_data();
  Serial.println("Zero Azimuth Calibration Complete");
}

void set_0_el_cal()
{
#ifdef debug
  Serial.println("Calibrate Zero Function");
#endif

  read_adc();
  // save current az and el values to EEPROM - Zero Calibration

#ifdef debug
  Serial.println(current_EL);
  Serial.println(current_AZ);
#endif

  EL_0 = current_EL;
  write_eeprom_cal_data();
  Serial.println("Zero Elevation Calibration Complete");
}

void read_adc()
{
#ifdef debug
  Serial.println("Read ADC Function  ");
#endif

  int RotorValue;
  adc.setGain(GAIN_TWOTHIRDS);
  delay(10); // adc settling delay
  current_EL = adc.readADC_SingleEnded(0);

#ifdef debug
  Serial.println(current_EL);
#endif

  delay(10); // adc settling delay
  current_AZ = adc.readADC_SingleEnded(1);

#ifdef debug
  Serial.println(current_AZ);
#endif

}

void check_move()
{
  if (set_AZ != -1 || set_EL != -1) {   // We're moving - check and stop as needed
    read_adc();
    // Map AZ to degrees

#ifdef debug
    Serial.println(current_AZ);
#endif

    AZ_Degrees = map(current_AZ, AZ_0, AZ_MAX, 0, 360);
    // Map EL to degrees

#ifdef debug
    Serial.println(current_EL);
#endif

    EL_Degrees = map(current_EL, EL_0, EL_MAX, 0, 180);

#ifdef debug
    Serial.println(EL_Degrees);
    Serial.println(AZ_Degrees);
#endif

    if (set_AZ != -1)
    {
      AZ_Distance = set_AZ - AZ_Degrees;
      if (abs(AZ_Distance) <= AZ_Tolerance) { // No move needed
        az_rotate_stop();
        set_AZ = -1;
      } else
      { // Move Azimuth - figure out which way
        if (AZ_Distance > 0) {  //We need to move CW
          rotate_az_cw();
        } else
        {
          rotate_az_ccw();
        }
      }
    }

    if (set_EL != -1)
    {
      EL_Distance = set_EL - EL_Degrees;

      if (abs(EL_Distance) <= EL_Tolerance)
      {
        // No move needed
        el_rotate_stop();
        set_EL = -1;
      } else
      { // Move Azimuth - figure out which way
        if (EL_Distance > 0)
        { //We need to move CW
          rotate_el_up();
        } else {
          rotate_el_down();
        }
      }
    }
  }
}

void start_tft()  // Starts the TFT display and shows the startup screen
{
  tft.begin();  // Start the TFT display

#ifdef debug1
  Serial.println("TFT Start");  // Verifies that the TFT is operational
  Serial.print("Display Pwr Mode: 0x");
  Serial.println(tft.readcommand8(ILI9341_RDMODE), HEX);
#endif

  clear_display();  // Clear the TFT display
  tft.setTextColor(text_color, background);  // Set the display text to black. Green background color required to "clear" previous text
  tft.setRotation(3); // horizontal display // change rotation as needed/desired
  tft.setTextSize(3);  // large font
  tft.setCursor(120, 50);  //Set the Cursor and display the startup screen
  tft.print("KW5GP");
  tft.setCursor(50, 90);
  tft.print("Yaesu GS-232A");
  tft.setCursor(50, 130);
  tft.print("Az/El Rotator");
  tft.setCursor(80, 170);
  tft.print("Controller");

  delay(5000);  //Wait 5 seconds then clear the startup message
  clear_display();  // Clear the TFT display
}

void clear_display()  // Function to clear the TFT display
{
  tft.fillScreen(background); // Clear the displayusing current background color
}

void update_tft() // Updates the current Azimuth and Elevation on the TFT display
{
  int Azimuth, Elevation;
  String s_Azimuth, s_Elevation;

  read_adc();
  // Map Azimuth to degrees
  Azimuth = map(current_AZ, AZ_0, AZ_MAX, 0, 360); // Map the AZ to 0 - 360
  // Adjust for North centered display
  if (Azimuth > 180)
  {
    Azimuth = Azimuth - 180;
  } else {
    Azimuth = Azimuth + 180;
  }

  // Map elevation to degrees
  Elevation = map(current_EL, EL_0, EL_MAX, 0, 180); // Map the EL to 0 - 180

  // Check to see if it's changed
  if (Elevation != prev_EL_Degrees || Azimuth != prev_AZ_Degrees)
  {
    // Position has changed - update TFT

    // Adjust to 3 characters each
    s_Azimuth = String(Azimuth);
    s_Elevation = String(Elevation);

    while (s_Azimuth.length() <3)
    {
      s_Azimuth = "0" + s_Azimuth;
    }

    while (s_Elevation.length() <3)
    {
      s_Elevation = "0" + s_Elevation;
    }
    
    tft.setTextSize(status_font_size);
    tft.setCursor(30, 150);
    tft.print("Elevation");
    tft.setCursor(210, 150);
    tft.print("Azimuth");

    tft.setTextSize(font_size);
    tft.setCursor(35, 90);
    tft.print(s_Elevation);
    tft.setCursor(200, 90);
    tft.print(s_Azimuth);


#ifdef debug1
    Serial.print("Azimuth: ");
    Serial.print(Azimuth);
    Serial.print("   Elevation:");
    Serial.print(Elevation);

    Serial.print("  AZ length ");
    Serial.print(s_Azimuth.length());
    Serial.print("   EZ length ");
    Serial.println(s_Elevation.length());
    delay(5000);
#endif

    prev_EL_Degrees = Elevation;
    prev_AZ_Degrees = Azimuth;

  }


}
